"""Utility to read headers from FITS files

This tool is intended to be used from a terminal in order to have easy access
FITS headers

Mode of Use:
    hselect.py image.fits keyword1 keyword2 ... keywordN
"""
from __future__ import absolute_import

from astropy.io import fits
import sys
import glob

__author__ = 'Simon Torres'
__date__ = '2016-11-09'
__version__ = "1.0"
__email__ = "storres@ctio.noao.edu"


class HeaderSelect(object):

    def __init__(self):
        """Defines the environment for getting keyword's values

        Also performs some checks on the validity of the input
        """
        self.args = sys.argv
        # print self.args
        try:
            if '*' not in self.args[1]:
                # if the program is called from a terminal
                self.file_list = [arg for arg in self.args if '.fits' in arg]
                self.keywords = [arg for arg in self.args[1:] if '.fits' not in arg]
            else:
                # if the program is called from pycharm
                self.file_list = glob.glob(self.args[1])
                self.keywords = self.args[2:]
            if self.file_list == []:
                print('Error: No images to get header')
                self.usage_exit()
            # if self.keywords == []:
                # print('Error: No keywords requested')
                # self.usage_exit()
        except IndexError:
            self.usage_exit()

    def __call__(self):
        """Get header and print results"""
        for image in self.file_list:
            image_info = "%s" % image
            try:
                header = fits.getheader(image)
                if self.keywords == []:
                    dimensions = [header['NAXIS%s' % i] for i in range(1, int(header['NAXIS']) + 1, 1)]
                    image_info += "%s: %s" %(dimensions, header['OBJECT'])
                else:
                    for key in self.keywords:
                        image_info += "\t%s" % header[key]
            except IOError:
                print("Image %s is corrupt or doesn't have a header" % image)
            except KeyError as err:
                image_info += "\t<KeyError %s>" % key.upper()
            print(image_info)

    @staticmethod
    def usage_exit():
        """Print usage and exit"""
        sys.exit('\nUsage: \n\thselect image.fits keyword1 keyword2 ... keywordN')


def header_select():
    hselect = HeaderSelect()
    hselect()


from astropy.io import fits
from ccdproc import CCDData
import glob
import os
import re


PATH = '/user/simon/development/soar/goodman_pipeline/goodman_pipeline/data/ref_comp'

keywords = {
    'SLIT': {'0.45" long slit': '0.45_LONG_SLIT'},
    'GRATING': {'SYZY_400':'400_SYZY',
                'SYZY_600-OLD':'600_SYZY_OLD',
                'SYZY_930':'930_SYZY',
                'RALC_1200-BLUE':'1200_RALC_BLUE'},

    'WAVMODE': 'wavmode',
}


def update_keywords(path):
    for _file in glob.glob(os.path.join(path, "*.fits")):
        ccd = CCDData.read(_file, unit='adu')
        print(_file)

        for key in keywords.keys():
            old_value = ccd.header[key]
            if key == 'WAVMODE':
                ccd.header[key] = re.sub(' ', '_', ccd.header[key]).upper()
            else:
                if ccd.header[key] in keywords[key].keys():
                    ccd.header[key] = keywords[key][ccd.header[key]]
            print("replaced {} with {}".format(old_value, ccd.header[key]))
        ccd.write(_file, overwrite=True)



if __name__ == '__main__':
    update_keywords(path=PATH)


from __future__ import absolute_import


def test_sample():
    assert 1 == 1

def test_import():
    try:
        from fits_utilities.hselect import HeaderSelect
        from fits_utilities.imhead import ImageHeader
    except ImportError:
        return False



import argparse
import sys
import glob
import matplotlib.pyplot as plt
import numpy as np
import re

from ccdproc import CCDData
from mpl_toolkits.axes_grid1 import make_axes_locatable


class ShowFits(object):

    def __init__(self):
        self.args = self.get_args()

        try:
            if '*' not in self.args.files:
                self.file_list = [arg for arg in self.args.files if '.fits' in arg]
                # self.keywords = [a]
            else:
                self.file_list = glob.glob(self.args.files)

            if self.file_list == []:
                print('Error')

        except IndexError:
            pass

    def __call__(self, *args, **kwargs):
        for file_name in self.file_list:
            ccd = CCDData.read(file_name, unit='adu')

            zlow, zhigh = self.__set_limits(ccd=ccd)

            if self.args.style == 'light':
                plt.style.use('default')
            elif self.args.style == 'dark':
                plt.style.use('dark_background')
            else:
                plt.style.use('dark_background')


            fig, ax = plt.subplots(figsize=(16, 9))
            fig.canvas.set_window_title(file_name)
            ax.set_title(file_name)
            im = ax.imshow(ccd.data, cmap=self.args.cmap, clim=(zlow, zhigh))
            divider = make_axes_locatable(ax)
            cax = divider.append_axes('right', size="3%", pad=0.05)
            fig.colorbar(im, cax=cax)
            plt.tight_layout()
            plt.show()

    @staticmethod
    def get_args():
        parser = argparse.ArgumentParser()
        parser.add_argument('files', help="File name or pattern to filter files")

        parser.add_argument('--color-map',
                            action='store',
                            default='viridis',
                            type=str, dest='cmap',
                            choices=['gray', 'viridis', 'gray_inverted'],
                            help='Color map to use')

        parser.add_argument('--style',
                            action='store',
                            default='dark',
                            type=str, dest='style',
                            choices=['dark', 'light'],
                            help='Visual style to use')

        args = parser.parse_args()
        if 'inverted' in args.cmap:
            args.cmap = re.sub('_inverted', '_r', args.cmap)
        return args

    @staticmethod
    def __set_limits(ccd):
        z1 = np.mean(ccd.data) - 0.5 * np.std(ccd.data)
        z2 = np.median(ccd.data) + np.std(ccd.data)

        return z1, z2


def show_fits():
    show__fits = ShowFits()
    show__fits()



import requests
import json
import os
import urllib.parse
import glob
import logging

from astropy.io import fits

logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

ACCESS_TOKEN = ''

SANDBOX_ZENODO_TOKEN = ''

zenodo_api_url = 'https://zenodo.org/api/'
sandobox_api_url = 'https://sandbox.zenodo.org/api/'


def upload_to_zenodo(path, file, meta, api_url, access_token):
    headers = {"Content-Type": "application/json"}
    params = {'access_token': access_token}
    r = requests.get(urllib.parse.urljoin(api_url, 'deposit/depositions'),
                     params=params)
    if r.status_code == 200:
        try:
            existing_files = [_item['files'][0]['filename'] for _item in r.json()]
            if file in existing_files:
                logger.error("File Exists")
        except KeyError:
            logger.error("No files uploaded yet.")

        rc = requests.post(
            urllib.parse.urljoin(api_url, 'deposit/depositions'),
            params=params,
            json={},
            headers=headers)
        if rc.status_code == 201:
            dep_id = rc.json()['id']
            data = {'filename': file}
            files = {'file': open(os.path.join(path, file), 'rb')}

            ru = requests.post(
                urllib.parse.urljoin(
                    api_url, 'deposit/depositions/{}/files'.format(dep_id)),
                params=params,
                data=data,
                files=files)
            logger.debug("Upload file status code: {}".format(ru.status_code))

            rmeta = requests.put(
                urllib.parse.urljoin(api_url,
                                     'deposit/depositions/{}'.format(dep_id)),
                params=params,
                data=json.dumps(meta),
                headers=headers)

            if rmeta.status_code == 200:
                r = requests.post(
                    urllib.parse.urljoin(
                        api_url,
                        'deposit/depositions/{}/actions/publish'.format(dep_id)),
                    params=params)

                if r.status_code == 202:
                    result = {'filename': file, 'id': r.json()['id']}
                    logger.info("Success: {}".format(result))
                    return result
                else:
                    logger.error("Publish failed, status code: {}"
                                 "".format(r.status_code))
            # print(ru.json())

        else:
            print("Returned status code: {}".format(rc.status_code))
            print(rc.json()['message'])





    else:
        print("API returned status code: {}".format(r.status_code))


def zenodo(path, sandbox=True):
    if sandbox:
        api_url = sandobox_api_url
        access_token = SANDBOX_ZENODO_TOKEN
    else:
        api_url = zenodo_api_url
        access_token = ACCESS_TOKEN

    all_results = []

    for _file in glob.glob(os.path.join(path, '*.fits')):
        print(_file)

        header = fits.getheader(_file)
        header_info = "Using the mode {} " \
                      "and slit {} " \
                      "exposed {} seconds".format(header['WAVMODE'],
                                                  header['SLIT'],
                                                  header['EXPTIME'])

        metadata = {
            'metadata': {
                'title': os.path.basename(_file),
                'upload_type': 'dataset',
                'description': 'Reference lamp library for the '
                               'Goodman Pipeline {}'.format(header_info),
                'creators': [{'name': 'Torres, Simon',
                              'affiliation': 'SOAR Telescope',
                              'orcid': '0000-0002-2726-6971'}],
                'keywords': ['spectroscopy', 'template', 'soar', 'telescope']
            }
        }
        result = upload_to_zenodo(path=path,
                                  file=os.path.basename(_file),
                                  meta=metadata,
                                  api_url=api_url,
                                  access_token=access_token)

        all_results.append(result)
    for _line in all_results:
        print(_line)


if __name__ == '__main__':
    zenodo(path='/user/simon/development/soar/goodman_pipeline/'
                'goodman_pipeline/data/ref_comp', sandbox=False)


import os

# Always prefer setuptools over distutils
from setuptools import setup, find_packages

# To use a consistent encoding
from codecs import open


here = os.path.abspath(os.path.dirname(__file__))


def create_version_py(packagename, version, source_dir='.'):
    package_dir = os.path.join(source_dir, packagename)
    version_py = os.path.join(package_dir, 'version.py')

    version_str = "# This is an automatic generated file please do not edit\n" \
                  "__version__ = '{:s}'".format(version)

    with open(version_py, 'w') as f:
        f.write(version_str)


# read content from README.md
with open(os.path.join(here, 'README.md')) as f:
    long_description = f.read()


# Get configuration information from setup.cfg
try:
    from ConfigParser import ConfigParser
except ImportError:
    from configparser import ConfigParser
conf = ConfigParser()


# conf.read([os.path.join(os.path.dirname(__file__), '..', 'setup.cfg')])
conf.read([os.path.join(os.path.dirname(__file__), 'setup.cfg')])
metadata = dict(conf.items('metadata'))

PACKAGENAME = metadata['package_name']

VERSION = metadata['version']

LICENSE = metadata['license']

DESCRIPTION = metadata['description']

LONG_DESCRIPTION = long_description

LONG_DESCRIPTION_CONTENT_TYPE = 'text/markdown'

AUTHOR = metadata['author']

AUTHOR_EMAIL = metadata['author_email']

INSTALL_REQUIRES = metadata['install_requires'].split()

# freezes version information in version.py
create_version_py(PACKAGENAME, VERSION)

setup(
    name=metadata['package_name'],

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=VERSION,

    description=DESCRIPTION,

    long_description=LONG_DESCRIPTION,

    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,

    # The project's main homepage.
    url='https://github.com/simontorres/fits_utilities',

    # Author details
    author=u'Simon Torres R., ',

    author_email='storres@ctio.noao.edu',

    # Choose your license
    license=LICENSE,

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',

        'License :: OSI Approved :: BSD License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.8',

        'Natural Language :: English',

        'Operating System :: POSIX :: Linux',
        'Operating System :: POSIX :: Other',
        'Operating System :: MacOS :: MacOS X',

        'Topic :: Scientific/Engineering :: Astronomy',
        'Topic :: Scientific/Engineering :: Information Analysis',
        'Topic :: Software Development :: Libraries :: Python Modules',

    ],

    # What does your project relate to?
    keywords='astronomy images',

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().

    packages=find_packages(),

    package_dir={'fits_utilities': 'fits_utilities'},



    install_requires=INSTALL_REQUIRES,

    entry_points={
        "console_scripts": [
            "showfits = fits_utilities.showfits:show_fits",
            "hselect = fits_utilities.hselect:header_select",
            "imhead = fits_utilities.imhead:image_header"]},

   )



"""Utility to read headers from FITS files

This tool is intended to be used from a terminal in order to have easy access
FITS headers.

If the arguments are only one image it will print the full header like IRAF's imheader
would do with the option longheader+ (l+). If you parse more than one image at a time it
will print the image name and the value of the keyword OBJECT


"""
from __future__ import absolute_import

from astropy.io import fits
import sys
import glob

__author__ = 'Simon Torres'
__date__ = '2016-12-26'
__version__ = "1.0"
__email__ = "storres@ctio.noao.edu"


class ImageHeader(object):

    def __init__(self):
        """Defines the environment for getting keyword's values

        Also performs some checks on the validity of the input
        """
        self.args = sys.argv
        # print self.args
        try:
            if '*' not in self.args[1]:
                # if the program is called from a terminal
                self.file_list = [arg for arg in self.args if '.fits' in arg]
                # self.keywords = [arg for arg in self.args[1:] if '.fits' not in arg]
            else:
                # if the program is called from pycharm
                self.file_list = glob.glob(self.args[1])
                # self.keywords = self.args[2:]
            if self.file_list == []:
                print('Error: No images to get header')
                self.usage_exit()
            # if self.keywords == []:
                # print('Error: No keywords requested')
                # self.usage_exit()
        except IndexError:
            self.usage_exit()

    def __call__(self):
        """Get header and print results"""
        for image in self.file_list:
            # image_info = "%s" % image
            try:
                header = fits.getheader(image)
                # print(header.keys)
                if len(self.file_list) == 1:
                    for keyword in header:
                        if keyword != 'COMMENT' and keyword != '':
                            print('{:8s}= {:30s} / {:47s}'.format(
                                str(keyword),
                                str(header[keyword]),
                                str(header.comments[keyword])))
                else:
                    print('{:35} {:15}'.format(image, header['OBJECT']))
            except IOError:
                print("Image %s is corrupt or doesn't have a header" % image)
            # except KeyError as err:
                # image_info += "\t<KeyError %s>" % key.upper()
            # print image_info

    @staticmethod
    def usage_exit():
        """Print usage and exit"""
        sys.exit('\nUsage: \n\timhead image.fits\n\timhead *fits')


def image_header():
    imhead = ImageHeader()
    imhead()

